from collections import namedtuple

ZeměInfo = namedtuple('ZeměInfo', 'plocha obyvatel hustota')

# A) načtení dat
lidi = {}
with open('staty_lidi.log') as soubor:
    for řádka in soubor:
        # vyčištění načtené textové řádky a její rozdělení na dva prvky
        stát, lidí = řádka.strip().split(': ')
        # uložení zjištěných údajů do slovníku
        lidi[stát] = lidí
print('Lidí:', len(lidi))    # 237

plochy = {}
with open('staty_plochy.log') as soubor:
    for řádka in soubor:
        # vyčištění načtené textové řádky a její rozdělení na dva prvky
        stát, plocha = řádka.strip().split(': ')
        # uložení zjištěných údajů do slovníku
        plochy[stát] = plocha
print('Ploch:', len(plochy))  # 261

# B) analýza dat
# ~ Zjistili jsme, že máme přesně o 24 údajů o plochách více než údajů o počtu obyvatel. Tudíž pro každý údaj s počtem obyvatel se můžeme bezpečně zeptat na odpovídající plochu.

# C) zpracování dat
hustoty = {}
# projdeme všech 261 záznamů o rozlohách zemí
for stát, plocha in plochy.items():
    # odpovídající počet obyvatel pro danou zem NEBO 0 pro neexistující záznam
    lidí = lidi.get(stát, 0)
    # výsledný slovník obsahující již i hustoty
    hustoty[stát] = ZeměInfo(
        plocha,                     # rozloha země v km^2
        lidí,                       # počet obyvatel
        int(lidí) / float(plocha),  # hustota zalidnění
    )
print(len(hustoty)) # 261
print(hustoty)
