from collections import namedtuple

CountryInfo = namedtuple('ZeměInfo', 'plocha obyvatel hustota')

# A) načtení dat
lidi = {}
with open('staty_lidi.log') as soubor:
    for řádka in soubor:
        # vyčištění načtené textové řádky a její rozdělení na dva prvky
        stát, lidí = řádka.strip().split(': ')
        # uložení zjištěných údajů do slovníku
        lidi[stát] = lidí

plochy = {}
with open('staty_plochy.log') as soubor:
    for řádka in soubor:
        # vyčištění načtené textové řádky a její rozdělení na dva prvky
        stát, plocha = řádka.strip().split(': ')
        # uložení zjištěných údajů do slovníku
        plochy[stát] = plocha

# B) analýza dat
# ~ Zjistili jsme, že máme přesně o 24 údajů o plochách více než údajů o počtu obyvatel. Tudíž pro každý údaj s počtem obyvatel se můžeme bezpečně zeptat na odpovídající plochu.

# C) zpracování dat
hustoty = {}
# projdeme všech 261 záznamů o rozlohách zemí
for stát, plocha in plochy.items():
    # odpovídající počet obyvatel pro danou zem NEBO 0 pro neexistující záznam
    lidí = lidi.get(stát, 0)
    # výsledný slovník obsahující již i hustoty
    hustoty[stát] = CountryInfo(
        plocha,                     # rozloha země v km^2
        lidí,                       # počet obyvatel
        int(lidí) / float(plocha),  # hustota zalidnění
    )
# zjištění nejdelšího názvu země
délky_názvů_zemí = {len(stát) for stát in hustoty}
print(len(délky_názvů_zemí))    # 29
print(délky_názvů_zemí)
'''
{4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 27, 30, 31, 32, 33, 35, 44}
'''
# výpis států podle hustoty zalidnění
# ~ přidáno pořadové číslo
# ~ info[-1] je druhý prvek z hustoty.items() neboli informace o dané zemi
# ~ PS: pro země ve slovníku 'lidi' je počet obyvatel řetězec, pro těch 24 chybějících je to číslo 0 (získané jako výstup z funkce get())
# POZOR: Při rozkladu na více řádek nezapomenout na čárky mezi jednotlivými parametry daných funkcí (zde postupně enumerate() a sorted())!
# ~ varianta bez koncových čárek u volání funkcí enumerate() a sorted()
for pořadí, (země, informace) in enumerate(
    # přes co cyklím
    sorted(
        # co řadíme (jsou to dvojce!)
        hustoty.items(),
        # podle čeho řadíme (hustota z druhého prvku dvojice)
        key=lambda info: info[-1].hustota,
        # výsledek vypíšeme v opačném pořadí
        reverse=True
    ),
    # od jakého čísla enumerate() začíná počítat
    start=1
):
    # rozbalení údajů o zemi do samostatných proměnných
    plocha, obyvatel, hustota = informace
    # výpis údajů o zemích
    print(f'{pořadí:>3} {země} --> {obyvatel} obyvatel na {plocha} km^2 => hustota {hustota:.0f}')
